from django import forms
from tenancy.models import Tenant
from ipam.models import Prefix, IPAddress
from dcim.models import Device
from virtualization.models import VirtualMachine
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField, DynamicModelMultipleChoiceField, TagFilterField
from .models import NASCluster, NASVolume, NASShare, NASMount, NASVolumeSecurityStyleChoices, NASShareTypeChoices, NASShareAccessLevelChoices

class NASClusterForm(NetBoxModelForm):
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    access_ips = DynamicModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False
    )
    comments = CommentField()
    class Meta:
        model = NASCluster
        fields = ('name', 'description', 'devices', 'tenant', 'tags', 'access_ips', 'comments')

class NASClusterFilterForm(NetBoxModelFilterSetForm):
    model = NASCluster
    name = forms.CharField(
        required=False
    )
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    tag = TagFilterField(model)

class NASVolumeForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = NASVolume
        fields = ('nas_cluster', 'name', 'export_id', 'owner', 'group', 'size_gb', 'local_directory', 'security_style', 'base_unix_permissions', 'description', 'tenant', 'tags', 'comments')

class NASVolumeFilterForm(NetBoxModelFilterSetForm):
    model = NASVolume
    nas_cluster = DynamicModelMultipleChoiceField(
        queryset=NASCluster.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )
    owner = forms.CharField(
        required=False
    )
    group = forms.CharField(
        required=False
    )
    security_style = forms.MultipleChoiceField(
        choices=NASVolumeSecurityStyleChoices,
        required=False
    )
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    tag = TagFilterField(model)

class NASShareForm(NetBoxModelForm):
    access_prefixes = DynamicModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False
    )
    access_ips = DynamicModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False
    )
    comments = CommentField()

    class Meta:
        model = NASShare
        fields = ('nas_volume', 'name', 'volume_subdirectory', 'type', 'access_level', 'access_prefixes', 'access_ips', 'mount_options', 'description', 'tenant', 'tags', 'comments')

class NASShareFilterForm(NetBoxModelFilterSetForm):
    model = NASShare
    nas_volume = DynamicModelMultipleChoiceField(
        queryset=NASVolume.objects.all(),
        required=False
    )
    name = forms.CharField(
        required=False
    )
    type = forms.MultipleChoiceField(
        choices=NASShareTypeChoices,
        required=False
    )
    access_level = forms.MultipleChoiceField(
        choices=NASShareAccessLevelChoices,
        required=False,
    )
    access_prefixes = DynamicModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False
    )
    access_ips = DynamicModelMultipleChoiceField(
        queryset=IPAddress.objects.all(),
        required=False
    )
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    tag = TagFilterField(model)

class NASMountForm(NetBoxModelForm):
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    virtual_machines = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False
    )
    comments = CommentField()

    class Meta:
        model = NASMount
        fields = ('nas_share', 'local_directory', 'devices', 'virtual_machines', 'mount_options', 'description', 'tenant', 'tags', 'comments')

class NASMountFilterForm(NetBoxModelFilterSetForm):
    model = NASMount
    nas_share = DynamicModelMultipleChoiceField(
        queryset=NASShare.objects.all(),
        required=False
    )
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    virtual_machines = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False
    )
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    tag = TagFilterField(model)



from netbox.filtersets import NetBoxModelFilterSet
from .models import NASCluster, NASVolume, NASShare, NASMount

class NASFilterSet(NetBoxModelFilterSet):
    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)

class NASClusterFilterSet(NASFilterSet):
    class Meta:
        model = NASCluster
        fields = ('id', 'name', 'tenant', 'tag', 'devices')

class NASVolumeFilterSet(NASFilterSet):
    class Meta:
        model = NASVolume
        fields = ('id', 'nas_cluster', 'name', 'local_directory', 'owner', 'group', 'security_style', 'tenant', 'tag')

class NASShareFilterSet(NASFilterSet):
    class Meta:
        model = NASShare
        fields = ('id', 'nas_volume', 'name', 'type', 'access_level', 'access_prefixes', 'access_ips', 'tenant', 'tag')

class NASMountFilterSet(NASFilterSet):
    class Meta:
        model = NASMount
        fields = ('id', 'nas_share', 'devices', 'virtual_machines', 'local_directory', 'tenant', 'tag')



from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel
from netbox.models.features import ContactsMixin
from tenancy.models import Tenant, ContactAssignment
from ipam.models import Prefix, IPAddress
from dcim.models import Device
from utilities.choices import ChoiceSet
from utilities.forms.fields import CommentField, DynamicModelChoiceField, DynamicModelMultipleChoiceField

class NASVolumeSecurityStyleChoices(ChoiceSet):
    CHOICES = [
        ('unix', 'UNIX'),
        ('windows', 'Windows'),
    ]

class NASShareTypeChoices(ChoiceSet):
    CHOICES = [
        ('nfs', 'NFS', 'orange'),
        ('smb', 'SMB/CIFS', 'green'),
        ('gpfs', 'GPFS', 'blue'),
        ('cephfs', 'CephFS', 'purple'),
    ]

class NASShareAccessLevelChoices(ChoiceSet):
    CHOICES = [
        ('rw', 'Read/Write'),
        ('ro', 'Read-Only'),
    ]

class NASCluster(ContactsMixin, NetBoxModel):
    name = models.CharField(
        max_length=100,
        unique=True
    )

    devices = models.ManyToManyField(
        to='dcim.Device',
        related_name='devices',
        blank=True,
        verbose_name='Devices'
    )

    access_ips = models.ManyToManyField(
        to='ipam.IPAddress',
        related_name='nas_cluster_access_ips',
        blank=True,
        verbose_name='Access IPs'
    )

    description = models.CharField(
        max_length=500,
        blank=True
    )

    comments = models.TextField(
        blank=True
    )

    tenant = models.ForeignKey(
        to=Tenant,
        on_delete=models.PROTECT,
        related_name='nas_clusters',
        blank = True,
        null = True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_nas:nascluster', args=[self.pk])

class NASVolume(ContactsMixin, NetBoxModel):
    nas_cluster = models.ForeignKey(
        to=NASCluster,
        on_delete=models.PROTECT,
        related_name='volumes'
    )

    name = models.CharField(
        max_length=100
    )

    export_id = models.PositiveIntegerField()

    owner = models.CharField(
        max_length=100
    )

    group = models.CharField(
        max_length=100
    )

    size_gb = models.PositiveIntegerField()

    local_directory = models.CharField(
        max_length=200
    )

    security_style = models.CharField(
        max_length=30,
        choices=NASVolumeSecurityStyleChoices,
        default='unix'
    )

    base_unix_permissions = models.CharField(
        max_length=100,
        default='2770'
    )

    description = models.CharField(
        max_length=500,
        blank=True
    )

    comments = models.TextField(
        blank=True
    )

    tenant = models.ForeignKey(
        to=Tenant,
        on_delete=models.PROTECT,
        related_name='nas_volumes',
        blank = True,
        null = True
    )

    class Meta:
        ordering = ('nas_cluster', 'local_directory')
        unique_together = (
            ('nas_cluster', 'local_directory'),
            ('nas_cluster', 'export_id'),
        )

    def __str__(self):
        return f'{self.nas_cluster}: {self.local_directory}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_nas:nasvolume', args=[self.pk])

class NASShare(ContactsMixin, NetBoxModel):
    nas_volume = models.ForeignKey(
        to=NASVolume,
        on_delete=models.PROTECT,
        related_name='shares'
    )

    name = models.CharField(
        max_length=50
    )

    volume_subdirectory = models.CharField(
        max_length=200,
        default='/'
    )

    type = models.CharField(
        max_length=30,
        choices=NASShareTypeChoices
    )

    mount_options = models.CharField(
        max_length=100,
        blank=True
    )

    access_level = models.CharField(
        max_length=30,
        choices=NASShareAccessLevelChoices,
        default='rw'
    )

    access_prefixes = models.ManyToManyField(
        to='ipam.Prefix',
        related_name='nas_share_access_prefixes',
        blank=True,
        verbose_name='Access Prefixes'
    )

    access_ips = models.ManyToManyField(
        to='ipam.IPAddress',
        related_name='nas_share_access_ips',
        blank=True,
        verbose_name='Access IPs'
    )

    description = models.CharField(
        max_length=500,
        blank=True
    )

    comments = models.TextField(
        blank=True
    )

    tenant = models.ForeignKey(
        to=Tenant,
        on_delete=models.PROTECT,
        related_name='nas_shares',
        blank = True,
        null = True
    )

    class Meta:
        ordering = ('nas_volume', 'name')
        unique_together = ('nas_volume', 'name')

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_nas:nasshare', args=[self.pk])

class NASMount(ContactsMixin, NetBoxModel):
    nas_share = models.ForeignKey(
        to=NASShare,
        on_delete=models.PROTECT,
        related_name='mounts'
    )

    devices = models.ManyToManyField(
        to='dcim.Device',
        related_name='nas_mount_devices',
        blank=True,
        verbose_name='Devices'
    )

    virtual_machines = models.ManyToManyField(
        to='virtualization.VirtualMachine',
        related_name='nas_mount_virtual_machines',
        blank=True,
        verbose_name='Virtual Machines'
    )

    local_directory = models.CharField(
        max_length=200
    )

    mount_options = models.CharField(
        max_length=100,
        blank=True
    )

    description = models.CharField(
        max_length=500,
        blank=True
    )

    comments = models.TextField(
        blank=True
    )

    tenant = models.ForeignKey(
        to=Tenant,
        on_delete=models.PROTECT,
        related_name='nas_mounts',
        blank = True,
        null = True
    )

    class Meta:
        ordering = ('nas_share', 'local_directory')
        unique_together = ('nas_share', 'local_directory')

    def __str__(self):
        return f'{self.nas_share}: {self.local_directory}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_nas:nasmount', args=[self.pk])



from django.db.models import Count
from netbox.views import generic
from tenancy.views import ObjectContactsView
from ipam.tables import PrefixTable, IPAddressTable
from dcim.tables import DeviceTable
from virtualization.tables import VirtualMachineTable
from utilities.views import register_model_view
from . import filtersets, forms, models, tables

# NAS Clusters
class NASClusterView(generic.ObjectView):
    queryset = models.NASCluster.objects.all()

    def get_extra_context(self, request, instance):
        volumes_table = tables.NASVolumeTable(instance.volumes.all())
        volumes_table.configure(request)
        devices_table = DeviceTable(instance.devices.all())
        devices_table.configure(request)
        access_ips_table = IPAddressTable(instance.access_ips.all())
        access_ips_table.configure(request)

        return {
            'volumes_table': volumes_table,
            'devices_table': devices_table,
            'access_ips_table': access_ips_table,
        }

@register_model_view(models.NASCluster, 'contacts')
class NASClusterContactsView(ObjectContactsView):
    queryset = models.NASCluster.objects.all()

class NASClusterListView(generic.ObjectListView):
    queryset = models.NASCluster.objects.all()
    table = tables.NASClusterTable
    filterset = filtersets.NASClusterFilterSet
    filterset_form = forms.NASClusterFilterForm

class NASClusterEditView(generic.ObjectEditView):
    queryset = models.NASCluster.objects.all()
    form = forms.NASClusterForm

class NASClusterDeleteView(generic.ObjectDeleteView):
    queryset = models.NASCluster.objects.all()


# NAS Volumes
class NASVolumeView(generic.ObjectView):
    queryset = models.NASVolume.objects.all()

    def get_extra_context(self, request, instance):
        shares_table = tables.NASShareTable(instance.shares.all())
        shares_table.configure(request)

        return {
            'shares_table': shares_table,
        }

@register_model_view(models.NASVolume, 'contacts')
class NASVolumeContactsView(ObjectContactsView):
    queryset = models.NASVolume.objects.all()

class NASVolumeListView(generic.ObjectListView):
    queryset = models.NASVolume.objects.all()
    table = tables.NASVolumeTable
    filterset = filtersets.NASVolumeFilterSet
    filterset_form = forms.NASVolumeFilterForm

class NASVolumeEditView(generic.ObjectEditView):
    queryset = models.NASVolume.objects.all()
    form = forms.NASVolumeForm

class NASVolumeDeleteView(generic.ObjectDeleteView):
    queryset = models.NASVolume.objects.all()


# NAS Shares
class NASShareView(generic.ObjectView):
    queryset = models.NASShare.objects.all()

    def get_extra_context(self, request, instance):
        mounts_table = tables.NASMountTable(instance.mounts.all())
        mounts_table.configure(request)
        prefixes_table = PrefixTable(instance.access_prefixes.all())
        prefixes_table.configure(request)
        ips_table = IPAddressTable(instance.access_ips.all())
        ips_table.configure(request)

        return {
            'mounts_table': mounts_table,
            'prefixes_table': prefixes_table,
            'ips_table': ips_table,
        }

@register_model_view(models.NASShare, 'contacts')
class NASShareContactsView(ObjectContactsView):
    queryset = models.NASShare.objects.all()

class NASShareListView(generic.ObjectListView):
    queryset = models.NASShare.objects.all()
    table = tables.NASShareTable
    filterset = filtersets.NASShareFilterSet
    filterset_form = forms.NASShareFilterForm

class NASShareEditView(generic.ObjectEditView):
    queryset = models.NASShare.objects.all()
    form = forms.NASShareForm

class NASShareDeleteView(generic.ObjectDeleteView):
    queryset = models.NASShare.objects.all()


# NAS Mounts
class NASMountView(generic.ObjectView):
    queryset = models.NASMount.objects.all()

    def get_extra_context(self, request, instance):
        devices_table = DeviceTable(instance.devices.all())
        devices_table.configure(request)
        virtual_machines_table = VirtualMachineTable(instance.virtual_machines.all())
        virtual_machines_table.configure(request)

        return {
            'devices_table': devices_table,
            'virtual_machines_table': virtual_machines_table,
        }

@register_model_view(models.NASMount, 'contacts')
class NASMountContactsView(ObjectContactsView):
    queryset = models.NASMount.objects.all()

class NASMountListView(generic.ObjectListView):
    queryset = models.NASMount.objects.all()
    table = tables.NASMountTable
    filterset = filtersets.NASMountFilterSet
    filterset_form = forms.NASMountFilterForm

class NASMountEditView(generic.ObjectEditView):
    queryset = models.NASMount.objects.all()
    form = forms.NASMountForm

class NASMountDeleteView(generic.ObjectDeleteView):
    queryset = models.NASMount.objects.all()



from netbox.plugins import PluginMenuButton, PluginMenuItem

nascluster_buttons = [
    PluginMenuButton(
        link='plugins:netbox_nas:nascluster_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
    ),
]
nasvolume_buttons = [
    PluginMenuButton(
        link='plugins:netbox_nas:nasvolume_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
    )
]
nasshare_buttons = [
    PluginMenuButton(
        link='plugins:netbox_nas:nasshare_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
    )
]
nasmount_buttons = [
    PluginMenuButton(
        link='plugins:netbox_nas:nasmount_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
    )
]

menu_items=(
    PluginMenuItem(
        link='plugins:netbox_nas:nascluster_list',
        link_text='NAS Clusters',
        buttons=nascluster_buttons
    ),
    PluginMenuItem(
        link='plugins:netbox_nas:nasvolume_list',
        link_text='NAS Volumes',
        buttons=nasvolume_buttons
    ),
    PluginMenuItem(
        link='plugins:netbox_nas:nasshare_list',
        link_text='NAS Shares',
        buttons=nasshare_buttons
    ),
    PluginMenuItem(
        link='plugins:netbox_nas:nasmount_list',
        link_text='NAS Mounts',
        buttons=nasmount_buttons
    )
)



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'netbox_nas'

router = NetBoxRouter()
router.register('nasclusters', views.NASClusterViewSet)
router.register('nasvolumes', views.NASVolumeViewSet)
router.register('nasshares', views.NASShareViewSet)
router.register('nasmounts', views.NASMountViewSet)

urlpatterns = router.urls



from netbox.search import SearchIndex, register_search
from .models import NASCluster, NASVolume, NASShare, NASMount

@register_search
class NASClusterIndex(SearchIndex):
  model = NASCluster
  fields = (
    ('name', 100),
    ('description', 500),
    ('comments', 500),
    ('tenant', 100),
  )

@register_search
class NASVolumeIndex(SearchIndex):
  model = NASVolume
  fields = (
    ('name', 100),
    ('export_id', 10),
    ('owner', 100),
    ('group', 100),
    ('local_directory', 500),
    ('description', 500),
    ('comments', 500),
    ('tenant', 100),
  )

@register_search
class NASShareIndex(SearchIndex):
  model = NASShare
  fields = (
    ('name', 100),
    ('volume_subdirectory', 500),
    ('description', 500),
    ('comments', 500),
    ('tenant', 100),
  )

@register_search
class NASMountIndex(SearchIndex):
  model = NASMount
  fields = (
    ('local_directory', 500),
    ('description', 500),
    ('comments', 500),
    ('tenant', 100),
  )



# Generated by Django 4.0.4 on 2022-04-26 03:27

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json

class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('tenancy', '0010_tenant_relax_uniqueness'),
        ('extras', '0073_journalentry_tags_custom_fields'),
        ('dcim', '0153_created_datetimefield'),
        ('virtualization', '0029_created_datetimefield'),
        ('ipam', '0057_created_datetimefield'),
    ]

    operations = [
        migrations.CreateModel(
            name='NASCluster',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('comments', models.TextField(blank=True)),
                ('devices', models.ManyToManyField(blank=True, related_name='devices', to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(help_text='A comma-separated list of tags.', through='extras.TaggedItem', to='extras.Tag', verbose_name='Tags')),
                ('access_ips', models.ManyToManyField(blank=True, related_name='nas_cluster_access_ips', to='ipam.ipaddress')),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='nas_clusters', to='tenancy.tenant')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='NASVolume',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('size_gb', models.PositiveIntegerField()),
                ('local_directory', models.CharField(max_length=200)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('comments', models.TextField(blank=True)),
                ('nas_cluster', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='volumes', to='netbox_nas.nascluster')),
                ('tags', taggit.managers.TaggableManager(help_text='A comma-separated list of tags.', through='extras.TaggedItem', to='extras.Tag', verbose_name='Tags')),
                ('base_unix_permissions', models.CharField(default='2770', max_length=100)),
                ('security_style', models.CharField(default='unix', max_length=30)),
                ('group', models.CharField(max_length=100)),
                ('owner', models.CharField(max_length=100)),
                ('name', models.CharField(max_length=100)),
                ('export_id', models.PositiveIntegerField()),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='nas_volumes', to='tenancy.tenant')),
            ],
            options={
                'ordering': ('nas_cluster', 'local_directory'),
                'unique_together': {('nas_cluster', 'local_directory'), ('nas_cluster', 'export_id')},
            },
        ),
        migrations.CreateModel(
            name='NASShare',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=50)),
                ('type', models.CharField(max_length=30)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('comments', models.TextField(blank=True)),
                ('nas_volume', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='shares', to='netbox_nas.nasvolume')),
                ('tags', taggit.managers.TaggableManager(help_text='A comma-separated list of tags.', through='extras.TaggedItem', to='extras.Tag', verbose_name='Tags')),
                ('mount_options', models.CharField(blank=True, max_length=100)),
                ('volume_subdirectory', models.CharField(default='/', max_length=200)),
                ('access_level', models.CharField(default='rw', max_length=30)),
                ('access_prefixes', models.ManyToManyField(blank=True, related_name='nas_share_access_prefixes', to='ipam.prefix')),
                ('access_ips', models.ManyToManyField(blank=True, related_name='nas_share_access_ips', to='ipam.ipaddress')),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='nas_shares', to='tenancy.tenant')),
            ],
            options={
                'ordering': ('nas_volume', 'name'),
                'unique_together': {('nas_volume', 'name')},
            },
        ),
        migrations.CreateModel(
            name='NASMount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('local_directory', models.CharField(max_length=200)),
                ('description', models.CharField(blank=True, max_length=500)),
                ('comments', models.TextField(blank=True)),
                ('devices', models.ManyToManyField(blank=True, related_name='nas_mount_devices', to='dcim.device')),
                ('nas_share', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='mounts', to='netbox_nas.nasshare')),
                ('tags', taggit.managers.TaggableManager(help_text='A comma-separated list of tags.', through='extras.TaggedItem', to='extras.Tag', verbose_name='Tags')),
                ('virtual_machines', models.ManyToManyField(blank=True, related_name='nas_mount_virtual_machines', to='virtualization.virtualmachine')),
                ('mount_options', models.CharField(blank=True, max_length=100)),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='nas_mounts', to='tenancy.tenant')),
            ],
            options={
                'ordering': ('nas_share', 'local_directory'),
                'unique_together': {('nas_share', 'local_directory')},
            },
        ),
    ]



from django.urls import path
from netbox.views.generic import ObjectChangeLogView, ObjectJournalView
from . import models, views

urlpatterns = (
    # NAS Clusters
    path('nascluster/', views.NASClusterListView.as_view(), name='nascluster_list'),
    path('nascluster/add/', views.NASClusterEditView.as_view(), name='nascluster_add'),
    path('nascluster/<int:pk>/', views.NASClusterView.as_view(), name='nascluster'),
    path('nascluster/<int:pk>/edit/', views.NASClusterEditView.as_view(), name='nascluster_edit'),
    path('nascluster/<int:pk>/delete/', views.NASClusterDeleteView.as_view(), name='nascluster_delete'),
    path('nascluster/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='nascluster_changelog', kwargs={
        'model': models.NASCluster
    }),
    path('nascluster/<int:pk>/journal/', ObjectJournalView.as_view(), name='nascluster_journal', kwargs={
        'model': models.NASCluster
    }),
    path('nascluster/<int:pk>/contacts/', views.NASClusterContactsView.as_view(), name='nascluster_contacts'),

    # NAS Volumes
    path('nasvolume/', views.NASVolumeListView.as_view(), name='nasvolume_list'),
    path('nasvolume/add/', views.NASVolumeEditView.as_view(), name='nasvolume_add'),
    path('nasvolume/<int:pk>/', views.NASVolumeView.as_view(), name='nasvolume'),
    path('nasvolume/<int:pk>/edit/', views.NASVolumeEditView.as_view(), name='nasvolume_edit'),
    path('nasvolume/<int:pk>/delete/', views.NASVolumeDeleteView.as_view(), name='nasvolume_delete'),
    path('nasvolume/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='nasvolume_changelog', kwargs={
        'model': models.NASVolume
    }),
    path('nasvolume/<int:pk>/journal/', ObjectJournalView.as_view(), name='nasvolume_journal', kwargs={
        'model': models.NASVolume
    }),
    path('nasvolume/<int:pk>/contacts/', views.NASVolumeContactsView.as_view(), name='nasvolume_contacts'),

    # NAS Shares
    path('nasshare/', views.NASShareListView.as_view(), name='nasshare_list'),
    path('nasshare/add/', views.NASShareEditView.as_view(), name='nasshare_add'),
    path('nasshare/<int:pk>/', views.NASShareView.as_view(), name='nasshare'),
    path('nasshare/<int:pk>/edit/', views.NASShareEditView.as_view(), name='nasshare_edit'),
    path('nasshare/<int:pk>/delete/', views.NASShareDeleteView.as_view(), name='nasshare_delete'),
    path('nasshare/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='nasshare_changelog', kwargs={
        'model': models.NASShare
    }),
    path('nasshare/<int:pk>/journal/', ObjectJournalView.as_view(), name='nasshare_journal', kwargs={
        'model': models.NASShare
    }),
    path('nasshare/<int:pk>/contacts/', views.NASShareContactsView.as_view(), name='nasshare_contacts'),

    # NAS Mounts
    path('nasmount/', views.NASMountListView.as_view(), name='nasmount_list'),
    path('nasmount/add/', views.NASMountEditView.as_view(), name='nasmount_add'),
    path('nasmount/<int:pk>/', views.NASMountView.as_view(), name='nasmount'),
    path('nasmount/<int:pk>/edit/', views.NASMountEditView.as_view(), name='nasmount_edit'),
    path('nasmount/<int:pk>/delete/', views.NASMountDeleteView.as_view(), name='nasmount_delete'),
    path('nasmount/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='nasmount_changelog', kwargs={
        'model': models.NASMount
    }),
    path('nasmount/<int:pk>/journal/', ObjectJournalView.as_view(), name='nasmount_journal', kwargs={
        'model': models.NASMount
    }),
    path('nasmount/<int:pk>/contacts/', views.NASMountContactsView.as_view(), name='nasmount_contacts'),
)



from rest_framework import serializers
from ipam.api.field_serializers import IPAddressField
from tenancy.api.serializers import NestedTenantSerializer
from ipam.api.serializers import NestedPrefixSerializer, NestedIPAddressSerializer
from dcim.api.serializers import NestedDeviceSerializer
from virtualization.api.serializers import NestedVirtualMachineSerializer
from ipam.models import Prefix, IPAddress
from dcim.models import Device
from virtualization.models import VirtualMachine
from netbox.api.fields import SerializedPKRelatedField
from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from ..models import NASCluster, NASVolume, NASShare, NASMount

class NestedNASIPDNSSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='ipam-api:ipaddress-detail'
    )
    address = IPAddressField()

    class Meta:
        model = IPAddress
        fields = ('id', 'url', 'address', 'dns_name')

class NestedNASClusterSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nascluster-detail'
    )
    access_ips = SerializedPKRelatedField(
        queryset=IPAddress.objects.all(),
        serializer=NestedNASIPDNSSerializer,
        required=False,
        many=True
    )

    class Meta:
        model = NASCluster
        fields = ('id', 'url', 'display', 'name', 'access_ips')

class NestedNASVolumeSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasvolume-detail'
    )

    class Meta:
        model = NASVolume
        fields = ('id', 'url', 'display', 'name', 'export_id', 'owner', 'group', 'size_gb', 'local_directory')

class NestedNASShareSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasshare-detail'
    )
    access_prefixes = SerializedPKRelatedField(
        queryset=Prefix.objects.all(),
        serializer=NestedPrefixSerializer,
        required=False,
        many=True
    )
    access_ips = SerializedPKRelatedField(
        queryset=IPAddress.objects.all(),
        serializer=NestedNASIPDNSSerializer,
        required=False,
        many=True
    )

    class Meta:
        model = NASShare
        fields = ('id', 'url', 'display', 'name', 'volume_subdirectory', 'mount_options', 'type', 'access_level', 'description', 'access_prefixes', 'access_ips')

class NestedNASMountSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasmount-detail'
    )

    class Meta:
        model = NASMount
        fields = ('id', 'url', 'display')

class NASClusterSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nascluster-detail'
    )
    devices = SerializedPKRelatedField(
        queryset=Device.objects.all(),
        serializer=NestedDeviceSerializer,
        required=False,
        many=True
    )
    access_ips = SerializedPKRelatedField(
        queryset=IPAddress.objects.all(),
        serializer=NestedNASIPDNSSerializer,
        required=False,
        many=True
    )
    tenant = NestedTenantSerializer()

    class Meta:
        model = NASCluster
        fields = ('id', 'url', 'display', 'name', 'devices', 'access_ips', 'description', 'tenant', 'tags', 'comments')

class NASVolumeSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasvolume-detail'
    )
    tenant = NestedTenantSerializer()
    nas_cluster = NestedNASClusterSerializer()
    shares = NestedNASShareSerializer(many=True)

    class Meta:
        model = NASVolume
        fields = ('id', 'url', 'display', 'name', 'export_id', 'owner', 'group', 'size_gb', 'local_directory', 'security_style', 'base_unix_permissions', 'description', 'tenant', 'tags', 'comments', 'nas_cluster', 'shares')

class NASShareSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasshare-detail'
    )
    tenant = NestedTenantSerializer()
    nas_volume = NestedNASVolumeSerializer
    access_prefixes = SerializedPKRelatedField(
        queryset=Prefix.objects.all(),
        serializer=NestedPrefixSerializer,
        required=False,
        many=True
    )
    access_ips = SerializedPKRelatedField(
        queryset=IPAddress.objects.all(),
        serializer=NestedNASIPDNSSerializer,
        required=False,
        many=True
    )

    class Meta:
        model = NASShare
        fields = ('id', 'url', 'display', 'name', 'type', 'mount_options', 'access_level', 'access_prefixes', 'access_ips', 'description', 'tenant', 'tags', 'comments', 'nas_volume')

class NASMountSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_nas-api:nasmount-detail'
    )
    nas_share = NestedNASShareSerializer()
    devices = SerializedPKRelatedField(
        queryset=Device.objects.all(),
        serializer=NestedDeviceSerializer,
        required=False,
        many=True
    )
    virtual_machines = SerializedPKRelatedField(
        queryset=VirtualMachine.objects.all(),
        serializer=NestedVirtualMachineSerializer,
        required=False,
        many=True
    )
    tenant = NestedTenantSerializer()

    class Meta:
        model = NASMount
        fields = ('id', 'url', 'display', 'devices', 'virtual_machines', 'local_directory', 'mount_options', 'description', 'tenant', 'tags', 'comments', 'nas_share')



from setuptools import find_packages, setup

setup(
    name='netbox-nas',
    version='1.0.2',
    description='Add NAS entities to NetBox',
    long_description_content_type='text/markdown',
    long_description=open('README.md', 'r').read(),
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)



from netbox.api.viewsets import NetBoxModelViewSet
from django.db.models import Count
from .. import filtersets, models
from .serializers import NASClusterSerializer, NASVolumeSerializer, NASShareSerializer, NASMountSerializer

class NASClusterViewSet(NetBoxModelViewSet):
    queryset = models.NASCluster.objects.prefetch_related('tags')
    serializer_class = NASClusterSerializer
    filterset_class = filtersets.NASClusterFilterSet

class NASVolumeViewSet(NetBoxModelViewSet):
    queryset = models.NASVolume.objects.prefetch_related('nas_cluster', 'tags')
    serializer_class = NASVolumeSerializer
    filterset_class = filtersets.NASVolumeFilterSet

class NASShareViewSet(NetBoxModelViewSet):
    queryset = models.NASShare.objects.prefetch_related('nas_volume', 'tags')
    serializer_class = NASShareSerializer
    filterset_class = filtersets.NASShareFilterSet

class NASMountViewSet(NetBoxModelViewSet):
    queryset = models.NASMount.objects.prefetch_related('nas_share', 'tags')
    serializer_class = NASMountSerializer
    filterset_class = filtersets.NASMountFilterSet



import django_tables2 as tables

from netbox.tables import NetBoxTable, ChoiceFieldColumn
from .models import NASCluster, NASVolume, NASShare, NASMount

class NASClusterTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    devices = tables.ManyToManyColumn(
        linkify_item=True
    )
    access_ips = tables.ManyToManyColumn(
        linkify_item=True
    )
    tenant = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = NASCluster
        fields = ('pk', 'id', 'name', 'devices', 'access_ips', 'tenant')
        default_columns = ('name', 'devices', 'access_ips', 'tenant')

class NASVolumeTable(NetBoxTable):
    nas_cluster = tables.Column(
        linkify=True
    )
    name = tables.Column(
        linkify=True
    )
    tenant = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = NASVolume
        fields = ('pk', 'id', 'name', 'export_id', 'owner', 'group', 'size_gb', 'local_directory', 'nas_cluster', 'security_style', 'base_unix_permissions', 'description', 'tenant')
        default_columns = ('name', 'export_id', 'owner', 'group', 'size_gb', 'local_directory', 'tenant', 'nas_cluster')

class NASShareTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    nas_volume = tables.Column(
        linkify=True
    )
    access_prefixes = tables.ManyToManyColumn(
        linkify_item=True
    )
    access_ips = tables.ManyToManyColumn(
        linkify_item=True
    )
    tenant = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = NASShare
        fields = ('pk', 'id', 'name', 'type', 'access_level', 'access_prefixes', 'access_ips', 'tenant', 'nas_volume')
        default_columns = ('name', 'type', 'access_level', 'nas_volume', 'tenant')

class NASMountTable(NetBoxTable):
    nas_share = tables.Column(
        linkify=True
    )
    devices = tables.ManyToManyColumn(
        linkify_item=True
    )
    virtual_machines = tables.ManyToManyColumn(
        linkify_item=True
    )
    tenant = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = NASMount
        fields = ('pk', 'id', 'local_directory', 'tenant', 'nas_share', 'devices', 'virtual_machines')
        default_columns = ('id', 'local_directory', 'tenant', 'nas_share', 'devices', 'virtual_machines')



from netbox.plugins import PluginConfig

class NetBoxNASConfig(PluginConfig):
    name = 'netbox_nas'
    verbose_name = 'NetBox NAS'
    description = 'Add NAS entities to NetBox'
    version = '1.0.2'
    base_url = 'nas'
    min_version = '4.0.0'

config = NetBoxNASConfig
